
from machine import Pin,SPI,PWM
import framebuf
import time
import os

# Pin definition
BL = 13
DC = 11
RST = 12
MOSI = 7
SCK = 6    
CS = 5
# color definition RBG
WHITE  =  0xFFFF
BLACK  =  0x0000
GREEN  =  0x001F
RED    =  0x07E0
BLUE   =  0xF800


class Lcd_1inch3():
    def __init__(self):
        self.w = 240
        self.h = 240
        
        self.cs = Pin(CS,Pin.OUT)
        self.rst = Pin(RST,Pin.OUT)
        self.dc = Pin(DC,Pin.OUT)
        self.cs(1)
        self.dc(1)        
        self.init_key()

        self.spi = SPI(0,16000_000,polarity=0, phase=0,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)

        self.buff = bytearray(self.h * self.w * 2) # set the lcd buffer size
        self.draw = framebuf.FrameBuffer(self.buff, self.w , self.h , framebuf.RGB565)
        self.init_lcd()
        
        
    def init_key(self):
        self.k1 = Pin(8,Pin.IN,Pin.PULL_UP) 
        self.k2 = Pin(4,Pin.IN,Pin.PULL_UP)
        self.k3 = Pin(3,Pin.IN,Pin.PULL_UP)
        self.k4 = Pin(2,Pin.IN,Pin.PULL_UP)
        self.left = Pin(16,Pin.IN,Pin.PULL_UP) 
        self.up = Pin(18,Pin.IN,Pin.PULL_UP)
        self.down = Pin(20,Pin.IN,Pin.PULL_UP)
        self.right = Pin(19,Pin.IN,Pin.PULL_UP)
        self.center = Pin(17,Pin.IN,Pin.PULL_UP)
        
    def write_cmd(self, cmd):    
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf):
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)

    def init_lcd(self):
        """Initialize lcd dispaly"""  
        self.rst(1)
        time.sleep(0.01)
        self.rst(0)
        time.sleep(0.01)
        self.rst(1)
        time.sleep(0.01)
        
        self.write_cmd(0x01) #software reset
        time.sleep(0.02)
        
        self.write_cmd(0x11)# sleep out 
        time.sleep(1)
        
        self.write_cmd(0x36) 
        self.write_data(0xA4)# MY=0,MX=1,MV=1,ML=0,RGB Order
        time.sleep(0.05)
        self.write_cmd(0x3A) 
        self.write_data(0x55) #65k-colors
        # FRMCTR1:Frame Rate Control in normal mode
        self.write_cmd(0xB2) # PORCTRL(Porch Setting)
        self.write_data(0x0C)
        self.write_data(0x0C)
        self.write_data(0x00)
        self.write_data(0x33)
        self.write_data(0x33)
        
        # FRMCTR2:Frame Rate Control in idle mode
        self.write_cmd(0xB7) # GCTRL:Gate control
        self.write_data(0x35)

        self.write_cmd(0xBB) # VCOMS:VCOM Setting
        self.write_data(0x37)
        
        self.write_cmd(0xC0) # LCMCTRL:LCM Control
        self.write_data(0x2C)
        
        self.write_cmd(0xC2) # VDVVRHEN:VDV and VRH Command Enable
        self.write_data(0x01)
        
        self.write_cmd(0xC3) # VRHS:VRH Set
        self.write_data(0x12)
        
        self.write_cmd(0xC4) # VDVS: VDV Set
        self.write_data(0x20)
        
        self.write_cmd(0xC6) # FRCTRL2:Frame Rate Control in Normal Mode
        self.write_data(0x0F) # 60Hz
        
        self.write_cmd(0xD0) # PWCTRL1:Power Control 1
        self.write_data(0xA4) # default
        self.write_data(0xA1) # 
        
        self.write_cmd(0xE0) # Positive Voltage Gamma Control
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0D)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2B)
        self.write_data(0x3F)
        self.write_data(0x54)
        self.write_data(0x4C)
        self.write_data(0x18)
        self.write_data(0x0D)
        self.write_data(0x0B)
        self.write_data(0x1F)
        self.write_data(0x23)
        
        self.write_cmd(0xE1) # NVGAMCTRL:Negative Voltage Gamma Control
        self.write_data(0xD4)
        self.write_data(0x04)
        self.write_data(0x0C)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2C)
        self.write_data(0x3F)
        self.write_data(0x44)
        self.write_data(0x51)
        self.write_data(0x2F)
        self.write_data(0x1F)
        self.write_data(0x1F)
        self.write_data(0x20)
        self.write_data(0x23)
        
        self.write_cmd(0x21)
        time.sleep(0.12)
        self.write_cmd(0x29)
        time.sleep(0.05)

    def set_windows(self, start_x, start_y, end_x, end_y):
        start_x += 80
        end_x += 80
        start_y += 0
        end_y += 0

        self.write_cmd(0x2A)  # 设置列地址
        self.write_data(start_x >> 8)
        self.write_data(start_x & 0xFF)
        self.write_data(end_x >> 8)
        self.write_data(end_x & 0xFF)

        self.write_cmd(0x2B)  # 设置行地址
        self.write_data(start_y >> 8)
        self.write_data(start_y & 0xFF)
        self.write_data(end_y >> 8)
        self.write_data(end_y & 0xFF)

        self.write_cmd(0x2C)  # 发送内存写入命令


    def show(self,buff):
        self.set_windows(0, 0, 239, 239) # 36H=0XEC:2, 3, 129, 130; 36H=0X68:1, 2, 128, 129         
        self.write_cmd(0x2C)
        
        self.dc(1)
        self.cs(0)
        self.spi.write(buff)
        self.cs(1)

  
if __name__=='__main__':
    bl = PWM(Pin(BL))
    bl.freq(1000)
    bl.duty_u16(49152) #75%, max 65535, 65536*75%=49152

    lcd = Lcd_1inch3()
    lcd.draw.fill(WHITE)
    lcd.draw.text('SEENGREAT!', 80, 60, RED)
    lcd.draw.text('PICO-1.3inchLCD', 60, 100, GREEN)
    lcd.draw.text('ABC123456789!', 70, 140, BLUE)
    lcd.draw.hline(1,1,239,BLUE)
    lcd.draw.vline(1,1,239,BLUE)
    lcd.draw.vline(238,238,239,BLUE)
    lcd.draw.vline(1,238,239,BLUE)
    lcd.draw.fill_rect(20,180,40,40,RED) #lcd_fill_rect(x, y, h, w, color)
    lcd.draw.fill_rect(100,180,40,40,GREEN)
    lcd.draw.fill_rect(180,180,40,40,BLUE)
    
    lcd.show(lcd.buff)
    time.sleep(5)
    lcd.draw.fill(WHITE)
    while(1):      
        if(lcd.k1.value() == 0):
            lcd.draw.fill_rect(180,30,45,30,GREEN)
        else :
            lcd.draw.fill_rect(180,30,45,30,BLACK)
            
        if(lcd.k2.value() == 0):
            lcd.draw.fill_rect(180,78,45,30,GREEN)
        else :
            lcd.draw.fill_rect(180,78,45,30,BLACK)
            
        if(lcd.k3.value() == 0):
            lcd.draw.fill_rect(180,126,45,30,GREEN)
        else :
            lcd.draw.fill_rect(180,126,45,30,BLACK)
            
        if(lcd.k4.value() == 0):  
            lcd.draw.fill_rect(180,174,45,30,GREEN)
        else :
            lcd.draw.fill_rect(180,174,45,30,BLACK)
#######################################################################
        if(lcd.right.value() == 0):  
            lcd.draw.fill_rect(110,96,20,40,GREEN) #右
        else :
            lcd.draw.fill_rect(110,96,20,40,BLACK)
#######################################################################           
        if(lcd.left.value() == 0):  
            lcd.draw.fill_rect(10,96,20,40,GREEN)  #左
        else :
            lcd.draw.fill_rect(10,96,20,40,BLACK)
#######################################################################           
        if(lcd.down.value() == 0):  
            lcd.draw.fill_rect(50,156,40,20,GREEN) #
        else :
            lcd.draw.fill_rect(50,156,40,20,BLACK)
#######################################################################       
        if(lcd.up.value() == 0):  
            lcd.draw.fill_rect(50,56,40,20,GREEN) #上
        else :
            lcd.draw.fill_rect(50,56,40,20,BLACK)
#######################################################################       
        if(lcd.center.value() == 0):  
            lcd.draw.fill_rect(50,96,40,40,GREEN)
        else :
            lcd.draw.fill_rect(50,96,40,40,BLACK)
#######################################################################
        lcd.show(lcd.buff)
    time.sleep(1)
    lcd.draw.fill(0xFFFF)







